@@ -227,6 +227,10 @@ struct cli_bc_ctx {
     cli_events_t *bc_events;
     int on_jit;
     int no_diff;
+#if HAVE_JSON
+    void **jsonobjs;
+    unsigned njsonobjs;
+#endif
 };
 struct cli_all_bc;
 int cli_vm_execute(const struct cli_bc *bc, struct cli_bc_ctx *ctx, const struct cli_bc_func *func, const struct cli_bc_inst *inst);
