@@ -30,10 +30,6 @@
 #include <unistd.h>
 #endif
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
 #include "clamav.h"
 #include "filetypes.h"
 #include "others.h"
@@ -45,6 +41,7 @@
 #include "iowrap.h"
 #include "mbr.h"
 #include "gpt.h"
+#include "ooxml.h"
 
 #include "htmlnorm.h"
 #include "entconv.h"
@@ -120,9 +117,13 @@ static const struct ftmap_s {
     { "CL_TYPE_OOXML_WORD",	CL_TYPE_OOXML_WORD     	},
     { "CL_TYPE_OOXML_PPT",	CL_TYPE_OOXML_PPT     	},
     { "CL_TYPE_OOXML_XL",	CL_TYPE_OOXML_XL     	},
+    { "CL_TYPE_INTERNAL",	CL_TYPE_INTERNAL     	},
+    { "CL_TYPE_XDP",        CL_TYPE_XDP             },
     { NULL,			CL_TYPE_IGNORED		}
 };
 
+cli_file_t cli_partitiontype(const unsigned char *buf, size_t buflen, const struct cl_engine *engine);
+
 cli_file_t cli_ftcode(const char *name)
 {
 	unsigned int i;
@@ -181,6 +182,7 @@ cli_file_t cli_partitiontype(const unsigned char *buf, size_t buflen, const stru
 	ptype = ptype->next;
     }
 
+    cli_dbgmsg("Partition type is potentially unsupported\n");
     return CL_TYPE_PART_ANY;
 }
 
@@ -266,10 +268,11 @@ cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine, cli_file_t
             const unsigned char * znamep = buff;
             int32_t zlen = bread;
             int lhc = 0;
-            int zi;
+            int zi, likely_ooxml = 0;
+            cli_file_t ret2;
             
             for (zi=0; zi<32; zi++) {
-                znamep = cli_memstr(znamep, zlen, lhdr_magic, 4);
+                znamep = (const unsigned char *)cli_memstr((const char *)znamep, zlen, lhdr_magic, 4);
                 if (NULL != znamep) {
                     znamep += SIZEOF_LH;
                     zlen = zread - (znamep - zbuff);
@@ -283,9 +286,34 @@ cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine, cli_file_t
                         } else if (0 == memcmp(znamep, "word/", 5)) {
                             cli_dbgmsg("Recognized OOXML Word file\n");
                             return CL_TYPE_OOXML_WORD;
+                        } else if (0 == memcmp(znamep, "docProps/", 5)) {
+                            likely_ooxml = 1;
+                        }
+
+                        if (++lhc > 2) {
+                            /* only check first three zip headers unless likely ooxml */
+                            if (likely_ooxml) {
+                                cli_dbgmsg("Likely OOXML, checking additional zip headers\n");
+                                if ((ret2 = cli_ooxml_filetype(NULL, map)) != CL_SUCCESS) {
+                                    /* either an error or retyping has occurred, return error or just CL_TYPE_ZIP? */
+                                    switch (ret2) {
+                                    case CL_TYPE_OOXML_XL:
+                                        cli_dbgmsg("Recognized OOXML XL file\n");
+                                        break;
+                                    case CL_TYPE_OOXML_PPT:
+                                        cli_dbgmsg("Recognized OOXML PPT file\n");
+                                        break;
+                                    case CL_TYPE_OOXML_WORD:
+                                        cli_dbgmsg("Recognized OOXML WORD file\n");
+                                        break;
+                                    default:
+                                        cli_dbgmsg("unexpected ooxml_filetype return: %i\n", ret2);
+                                    }
+                                    return ret2;
+                                }
+                            }
+                            break;
                         }
-                        if (++lhc > 2)
-                            break; /* only check first three zip headers */
                     }
                     else {
                         znamep = NULL; /* force to map more */
