@@ -52,10 +52,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
-
+#include "clamav.h"
 #include "cltypes.h"
 #include "others.h"
 #include "upx.h"
@@ -225,6 +222,10 @@ static int pefromupx (const char *src, uint32_t ssize, char *dst, uint32_t *dsiz
     cli_writeint32(sections+12, urva);
     cli_writeint32(sections+16, vsize);
     cli_writeint32(sections+20, foffset);
+    if (foffset + vsize < foffset) {
+        /* Integer overflow */
+        return 0;
+    }
     foffset+=vsize;
     
     sections+=0x28;
@@ -242,7 +243,20 @@ static int pefromupx (const char *src, uint32_t ssize, char *dst, uint32_t *dsiz
   memcpy(newbuf+0xd0, pehdr,0xf8+0x28*sectcnt);
   sections = pehdr+0xf8;
   for (upd = 0; upd <sectcnt ; upd++) {
-    memcpy(newbuf+cli_readint32(sections+20), dst+cli_readint32(sections+12)-upx0, cli_readint32(sections+16));
+      uint32_t offset1, offset2, offset3;
+      offset1 = (uint32_t)cli_readint32(sections+20);
+      offset2 = (uint32_t)cli_readint32(sections+16);
+      if (offset1 > foffset || offset2 > foffset || offset1 + offset2 > foffset) {
+          free(newbuf);
+          return 1;
+      }
+
+      offset3 = (uint32_t)cli_readint32(sections+12);
+      if (offset3-upx0 > *dsize) {
+          free(newbuf);
+          return 1;
+      }
+    memcpy(newbuf+offset1, dst+offset3-upx0, offset2);
     sections+=0x28;
   }
 
